@@ -203,13 +203,12 @@ struct CausalMask : NoMask {
 
     // See note below on different ways to think about causal attention
     // Again, we'd add the offset_q into the max_blocks_q calculation
+    int max_blocks_k = Base::get_trip_count(blk_coord, tile_shape, problem_size);
     if constexpr (IsQBegin) {
-      int max_blocks_k = Base::get_trip_count(blk_coord, tile_shape, problem_size);
       int max_blocks_q = ceil_div((get<0>(blk_coord) + 1) * get<0>(tile_shape), get<1>(tile_shape));
       return std::min(max_blocks_k, max_blocks_q);
     } else {
       const int offset_q = get<1>(problem_size) - get<0>(problem_size);
-      int max_blocks_k = Base::get_trip_count(blk_coord, tile_shape, problem_size);
       int max_blocks_q = ceil_div((get<0>(blk_coord) + 1) * get<0>(tile_shape) + offset_q, get<1>(tile_shape));
       return std::min(max_blocks_k, max_blocks_q);
     }
@@ -222,12 +221,12 @@ struct CausalMask : NoMask {
       TileShape const& tile_shape,
       ProblemSize const& problem_size) {
         
+    int trip_count = get_trip_count(blk_coord, tile_shape, problem_size);
     if constexpr (IsQBegin) {
-      int trip_count = get_trip_count(blk_coord, tile_shape, problem_size);
       return std::min(trip_count, int(ceil_div(size<0>(tile_shape), size<1>(tile_shape))));
     } else {
-      const int offset_tile_q = get<1>(problem_size) % get<1>(tile_shape);
-      return ceil_div(get<0>(tile_shape) +  offset_tile_q, get<1>(tile_shape));
+      const int offset_tile_q = (get<1>(problem_size) - get<0>(problem_size)) % get<1>(tile_shape);
+      return std::min(trip_count, int(ceil_div(get<0>(tile_shape) + offset_tile_q, get<1>(tile_shape))));
     }
   }
 
@@ -277,9 +276,10 @@ struct CausalMask : NoMask {
   }
 };
 
-struct CausalForBackwardMask : CausalMask<true>, ResidualMaskForBackward {
+template<bool kIsQBegin = true>
+struct CausalForBackwardMask : CausalMask<kIsQBegin>, ResidualMaskForBackward {
 
-  using Base = CausalMask<true>;
+  using Base = CausalMask<kIsQBegin>;
 
   template<class AccQK, class IndexQK, class ProblemSize>
   CUTLASS_DEVICE
@@ -296,10 +296,15 @@ struct CausalForBackwardMask : CausalMask<true>, ResidualMaskForBackward {
     //      where we only compute the next row and use cache for the rest
     //    - if you'd like this, you only need to add an offset like so:
     //      get<0>(pos) + offset_q < get<1>(pos)
+    int offset_q = 0;
+    if constexpr (!kIsQBegin) {
+      offset_q = get<1>(problem_size) - get<0>(problem_size);
+    }
+
     CUTLASS_PRAGMA_UNROLL
     for (int i = 0; i < size(acc_qk); i++) {
       auto pos = index_qk(i);
-      bool masked = (get<0>(pos) < get<1>(pos)) || !elem_less(pos, problem_size);
+      bool masked = (get<0>(pos) + offset_q < get<1>(pos)) || !elem_less(pos, problem_size);
       if (masked) {
         acc_qk(i) = -INFINITY;
       }
