@@ -94,7 +94,7 @@ public:
   };
 
   using ProblemShape = typename GroupProblemShape::UnderlyingProblemShape;
-  using Params = PersistentTileSchedulerSm90GroupParams<ProblemShape>;
+  using Params = PersistentTileSchedulerSm90GroupParams<GroupProblemShape>;
   using RasterOrder = typename Params::RasterOrder;
   using RasterOrderOptions = typename Params::RasterOrderOptions;
   static constexpr bool IsDynamicPersistent = false;
@@ -160,7 +160,6 @@ public:
     static_assert(cute::is_static<ClusterShape>::value);
 
     dim3 problem_blocks = get_tiled_cta_shape_mnl(
-      problem_shapes.groups(),
       problem_shapes,
       hw_info,
       tile_shape, cluster_shape);
@@ -168,9 +167,7 @@ public:
     Params params;
     params.initialize(
       problem_blocks,
-      problem_shapes.groups(),
-      problem_shapes.problem_shapes,
-      problem_shapes.host_problem_shapes,
+      problem_shapes,
       to_gemm_coord(tile_shape),
       to_gemm_coord(cluster_shape),
       hw_info,
@@ -187,7 +184,7 @@ public:
   dim3
   get_grid_shape(
     [[maybe_unused]] Params const& params,
-    GroupProblemShape problem_shapes,
+    GroupProblemShape const& problem_shapes,
     TileShape tile_shape,
     ClusterShape cluster_shape,
     KernelHardwareInfo hw_info,
@@ -195,7 +192,6 @@ public:
     bool truncate_by_problem_size=true) {
 
     dim3 problem_blocks = get_tiled_cta_shape_mnl(
-      problem_shapes.groups(),
       problem_shapes,
       hw_info,
       tile_shape, cluster_shape);
@@ -215,7 +211,8 @@ public:
   template<class BlockShape, class ClusterShape>
   CUTLASS_HOST_DEVICE static
   dim3
-  get_tiled_cta_shape_mnl(int groups, GroupProblemShape problem_shapes, KernelHardwareInfo hw_info, BlockShape cta_shape, ClusterShape cluster_shape) {
+  get_tiled_cta_shape_mnl(GroupProblemShape const& problem_shapes, KernelHardwareInfo hw_info, BlockShape cta_shape, ClusterShape cluster_shape) {
+    int groups = problem_shapes.groups();
     uint32_t total_ctas = 0;
     uint32_t cta_in_N_dim = 1; // We linearize the blocks across all the problems here
 
@@ -259,20 +256,21 @@ public:
     }
 
     int lane_idx = canonical_lane_idx();
-    if (lane_idx < params_.groups_) {
-      cached_problem_shapes_[1] = params_.problem_shapes_[lane_idx];
+    if (lane_idx < params_.problem_shapes_.groups()) {
+      cached_problem_shapes_[1] = params_.problem_shapes_.get_problem_shape(lane_idx);
     }
 
     total_grid_size_ = uint64_t(gridDim.x) * uint64_t(gridDim.y) * uint64_t(gridDim.z);
     uint64_t ctas_along_m, ctas_along_n;
-    if (is_tuple<decltype(cute::shape<0>(params_.problem_shapes_[0]))>::value ||
-        is_tuple<decltype(cute::shape<1>(params_.problem_shapes_[0]))>::value) {
-      ctas_along_m = cute::size(cute::ceil_div(cute::shape<0>(params_.problem_shapes_[0]), scheduler_params.cta_shape_.m()));
-      ctas_along_n = cute::size(cute::ceil_div(cute::shape<1>(params_.problem_shapes_[0]), scheduler_params.cta_shape_.n()));
+    ProblemShape problem_shape = params_.problem_shapes_.get_problem_shape(0);
+    if (is_tuple<decltype(cute::shape<0>(problem_shape))>::value ||
+        is_tuple<decltype(cute::shape<1>(problem_shape))>::value) {
+      ctas_along_m = cute::size(cute::ceil_div(cute::shape<0>(problem_shape), scheduler_params.cta_shape_.m()));
+      ctas_along_n = cute::size(cute::ceil_div(cute::shape<1>(problem_shape), scheduler_params.cta_shape_.n()));
     }
     else {
-      ctas_along_m = scheduler_params.divmod_cta_shape_m_.divide(cute::shape<0>(params_.problem_shapes_[0]) +  scheduler_params.divmod_cta_shape_m_.divisor - 1);
-      ctas_along_n = scheduler_params.divmod_cta_shape_n_.divide(cute::shape<1>(params_.problem_shapes_[0]) +  scheduler_params.divmod_cta_shape_n_.divisor - 1);
+      ctas_along_m = scheduler_params.divmod_cta_shape_m_.divide(cute::shape<0>(problem_shape) +  scheduler_params.divmod_cta_shape_m_.divisor - 1);
+      ctas_along_n = scheduler_params.divmod_cta_shape_n_.divide(cute::shape<1>(problem_shape) +  scheduler_params.divmod_cta_shape_n_.divisor - 1);
     }
     auto problem_blocks_m = round_up(ctas_along_m, (1 << params_.log_swizzle_size_) * params_.cluster_shape_.m());
     auto problem_blocks_n = round_up(ctas_along_n, (1 << params_.log_swizzle_size_) * params_.cluster_shape_.n());
@@ -292,8 +290,7 @@ public:
   get_work_idx_m_and_n(
       uint64_t linear_idx,
       GroupInfo& group_info,
-      int32_t total_problem_groups,
-      ProblemShape* problem_shapes,
+      GroupProblemShape &problem_shapes,
       ProblemShape (&cached_problem_shapes)[2],
       GemmCoord cta_shape,
       GemmCoord cluster_shape,
@@ -308,13 +305,14 @@ public:
 
     // Use a warp to "speculatively" check if the work tile maps to the next 32 groups
     int lane_idx = canonical_lane_idx();
+    int total_problem_groups = problem_shapes.groups();
 
     if (linear_idx >= group_info.total_tiles + group_info.start_linear_idx) {
       group_info.group_idx += lane_idx;
       for ( ; ; group_info.group_idx += NumThreadsPerWarp) {
         cached_problem_shapes[0] = cached_problem_shapes[1];
         if (group_info.group_idx + NumThreadsPerWarp < total_problem_groups) {
-          cached_problem_shapes[1] = problem_shapes[group_info.group_idx + NumThreadsPerWarp];
+          cached_problem_shapes[1] = problem_shapes.get_problem_shape(group_info.group_idx + NumThreadsPerWarp);
         }
         if (group_info.group_idx < total_problem_groups) {
           uint64_t ctas_along_m, ctas_along_n;
@@ -354,7 +352,7 @@ public:
           group_info.total_tiles = __shfl_sync(0xffffffff, group_info.total_tiles, first_succeeding_thread);
           group_info.problem_blocks_along_raster_order = __shfl_sync(0xffffffff, group_info.problem_blocks_along_raster_order, first_succeeding_thread);
           if (group_info.group_idx + lane_idx < total_problem_groups) {
-            cached_problem_shapes[1] = problem_shapes[group_info.group_idx + lane_idx];
+            cached_problem_shapes[1] = problem_shapes.get_problem_shape(group_info.group_idx + lane_idx);
           }
           break;
         }
@@ -419,7 +417,6 @@ public:
     return get_work_idx_m_and_n<WorkTileInfo>(
               linear_idx,
               current_group_info_,
-              scheduler_params.groups_,
               scheduler_params.problem_shapes_,
               cached_problem_shapes_,
               scheduler_params.cta_shape_,
